@@ -76,7 +76,7 @@ setMethod("Ops",
 setMethod("Ops",
           c(e1 = "fmpz", e2 = "fmpz"),
           function (e1, e2)
-              .Call(R_flint_fmpz_ops2, .Generic, e1, e2, list()))
+              .Call(R_flint_fmpz_ops2, .Generic, e1, e2, NULL))
 
 setMethod("Ops",
           c(e1 = "fmpz", e2 = "fmpq"),
@@ -249,6 +249,19 @@ setMethod("determinant",
                         "det")
           })
 
+setMethod("diff",
+          c(x = "fmpz"),
+          function (x, lag = 1L, differences = 1L, ...)
+              .Call(R_flint_fmpz_ops1, "diff", x,
+                    list(as.integer(lag), as.integer(differences))))
+
+setMethod("diffinv",
+          c(x = "fmpz"),
+          function (x, lag = 1L, differences = 1L, xi, ...)
+              .Call(R_flint_fmpz_ops1, "diffinv", x,
+                    list(as.integer(lag), as.integer(differences),
+                         if (!missing(xi)) as(xi, "fmpz"))))
+
 setMethod("format",
           c(x = "fmpz"),
           function (x, base = 10L, ...)
@@ -279,6 +292,18 @@ setMethod("is.unsorted",
           function (x, na.rm = FALSE, strictly = FALSE)
               .Call(R_flint_fmpz_ops1, "is.unsorted", x, list(NULL, as.logical(strictly))))
 
+setMethod("isComplex",
+          c(x = "fmpz"),
+          function (x) FALSE)
+
+setMethod("isFloating",
+          c(x = "fmpz"),
+          function (x) FALSE)
+
+setMethod("isSigned",
+          c(x = "fmpz"),
+          function (x) TRUE)
+
 setMatrixOpsMethod(
           c(x = "ANY", y = "fmpz"),
           function (x, y) {
@@ -299,7 +324,7 @@ setMatrixOpsMethod(
           c(x = "fmpz", y = "ANY"),
           function (x, y) {
               if (.Generic != "%*%" && (missing(y) || is.null(y)))
-                  return(.Call(R_flint_fmpz_ops2, .Generic, x, x, list()))
+                  return(.Call(R_flint_fmpz_ops2, .Generic, x, x, NULL))
               g <- get(.Generic, mode = "function")
               switch(typeof(y),
                      "NULL" =, "raw" =, "logical" =, "integer" =
@@ -326,7 +351,7 @@ setMatrixOpsMethod(
 setMatrixOpsMethod(
           c(x = "fmpz", y = "fmpz"),
           function (x, y)
-              .Call(R_flint_fmpz_ops2, .Generic, x, y, list()))
+              .Call(R_flint_fmpz_ops2, .Generic, x, y, NULL))
 
 setMatrixOpsMethod(
           c(x = "fmpz", y = "fmpq"),
@@ -394,7 +419,7 @@ setMethod("solve",
           c(a = "fmpz", b = "ANY"),
           function (a, b, ...) {
               if (missing(b))
-                  return(.Call(R_flint_fmpz_ops1, "solve", a, list()))
+                  return(.Call(R_flint_fmpz_ops1, "solve", a, NULL))
               switch(typeof(b),
                      "NULL" =, "raw" =, "logical" =, "integer" =
                          solve(a, fmpz(b), ...),
@@ -420,7 +445,7 @@ setMethod("solve",
 setMethod("solve",
           c(a = "fmpz", b = "fmpz"),
           function (a, b, ...)
-              .Call(R_flint_fmpz_ops2, "solve", a, b, list()))
+              .Call(R_flint_fmpz_ops2, "solve", a, b, NULL))
 
 setMethod("solve",
           c(a = "fmpz", b = "fmpq"),
