@@ -49,26 +49,6 @@ do { \
 #define arf_conj(rop, op) arf_set(rop, op)
 #define arb_conj(rop, op) arb_set(rop, op)
 
-#ifndef HAVE_ACF_ZERO
-static R_INLINE
-void acf_zero(acf_t res)
-{
-	arf_zero(acf_realref(res));
-	arf_zero(acf_imagref(res));
-	return;
-}
-#endif
-
-#ifndef HAVE_ACF_CONJ
-static R_INLINE
-void acf_conj(acf_t res, const acf_t x)
-{
-	arf_set(acf_realref(res), acf_realref(x));
-	arf_neg(acf_imagref(res), acf_imagref(x));
-	return;
-}
-#endif
-
 #if __FLINT_RELEASE < 30100
 FLINT_NORETURN
 #endif
@@ -425,7 +405,7 @@ SEXP R_flint_bind(SEXP s_op, SEXP s_usenames, SEXP args, SEXP exps)
 			elt = VECTOR_ELT(args, jargs);
 			nx = R_flint_get_length(elt);
 			if (nx > UWORD_MAX - ny)
-				Rf_error(_("value length would exceed maximum %llu"),
+				Rf_error(_("length would exceed maximum %llu"),
 				         (unsigned long long int) UWORD_MAX);
 			if (!anynames &&
 			    (TAG(e) != R_NilValue ||
@@ -481,7 +461,13 @@ SEXP R_flint_bind(SEXP s_op, SEXP s_usenames, SEXP args, SEXP exps)
 			} else {
 				sa = Rf_translateCharUTF8(PRINTNAME(TAG(e)));
 				la = strlen(sa);
-				if (namesx == R_NilValue)
+				if (namesx == R_NilValue && nx == 1) {
+					snprintf(buf, la + 1, "%s", sa);
+					SET_STRING_ELT(namesy, (R_xlen_t) jy,
+					               Rf_mkCharCE(buf, CE_UTF8));
+					jy += 1;
+				}
+				else if (namesx == R_NilValue)
 					for (jx = 0; jx < nx; ++jx, ++jy) {
 						lb = (size_t) log10((double) (jx + 1)) + 1;
 						snprintf(buf, la + lb + 1, "%s%llu",
@@ -493,7 +479,7 @@ SEXP R_flint_bind(SEXP s_op, SEXP s_usenames, SEXP args, SEXP exps)
 					for (jx = 0; jx < nx; ++jx, ++jy) {
 						sb = Rf_translateCharUTF8(STRING_ELT(namesx, (R_xlen_t) jx));
 						lb = strlen(sb);
-						snprintf(buf, la + lb + 1, "%s.%s",
+						snprintf(buf, la + 1 + lb + 1, "%s.%s",
 						         sa, sb);
 						SET_STRING_ELT(namesy, (R_xlen_t) jy,
 						               Rf_mkCharCE(buf, CE_UTF8));
@@ -562,8 +548,8 @@ SEXP R_flint_bind(SEXP s_op, SEXP s_usenames, SEXP args, SEXP exps)
 				         INT_MAX);
 			dy[ op] += (int) nnull;
 		}
-		if ((unsigned int) dy[0] > UWORD_MAX / (unsigned int) dy[1])
-			Rf_error(_("value length would exceed maximum %llu"),
+		if (dy[0] > UWORD_MAX / (mp_limb_t) dy[1])
+			Rf_error(_("length would exceed maximum %llu"),
 			         (unsigned long long int) UWORD_MAX);
 		ny = (mp_limb_t) dy[0] * (mp_limb_t) dy[1];
 
@@ -610,17 +596,17 @@ SEXP R_flint_bind(SEXP s_op, SEXP s_usenames, SEXP args, SEXP exps)
 						for (jx = 0; jx < nx; ++jx, ++jy) \
 							name##_set(y__ + jy, x__ + jx); \
 					} else { \
-						for (j = 0, jx = 0; j < dx[1]; ++j, jy += dy[0] - dx[0]) \
+						for (j = 0, jx = 0; j < dx[1]; ++j, jy += (mp_limb_t) (dy[0] - dx[0])) \
 							for (i = 0; i < dx[0]; ++i, ++jx, ++jy) \
 								name##_set(y__ + jy, x__ + jx); \
-						jy -= ny - dx[0]; \
+						jy -= ny - (mp_limb_t) dx[0]; \
 					} \
 				} else if ((nx = R_flint_get_length(elt)) == dy[!op]) { \
 					if (op == 1) \
 						for (jx = 0; jx < nx; ++jx, ++jy) \
 							name##_set(y__ + jy, x__ + jx); \
 					else { \
-						for (jx = 0; jx < nx; ++jx, jy += dy[op]) \
+						for (jx = 0; jx < nx; ++jx, jy += (mp_limb_t) dy[op]) \
 							name##_set(y__ + jy, x__ + jx); \
 						jy -= ny - 1; \
 					} \
@@ -629,21 +615,21 @@ SEXP R_flint_bind(SEXP s_op, SEXP s_usenames, SEXP args, SEXP exps)
 						for (i = 0; i < dy[0]; ++i, ++jy) \
 							name##_set(y__ + jy, x__); \
 					else { \
-						for (j = 0; j < dy[1]; ++j, jy += dy[0]) \
+						for (j = 0; j < dy[1]; ++j, jy += (mp_limb_t) dy[0]) \
 							name##_set(y__ + jy, x__); \
 						jy -= ny - 1; \
 					} \
 				} else if (nx > 0) { \
 					/* FIXME? memory is leaked if warning is caught */ \
 					if (op == 1) { \
-						if (dy[0] % nx) \
+						if ((mp_limb_t) dy[0] % nx) \
 							Rf_warning(_("number of rows of return value is not a multiple of vector argument length")); \
 						for (i = 0, jx = 0; i < dy[0]; ++i, jx = (++jx == nx) ? 0 : jx, ++jy) \
 							name##_set(y__ + jy, x__ + jx); \
 					} else { \
-						if (dy[1] % nx) \
+						if ((mp_limb_t) dy[1] % nx) \
 							Rf_warning(_("number of columns of return value is not a multiple of vector argument length")); \
-						for (j = 0, jx = 0; j < dy[1]; ++j, jx = (++jx == nx) ? 0 : jx, jy += dy[0]) \
+						for (j = 0, jx = 0; j < dy[1]; ++j, jx = (++jx == nx) ? 0 : jx, jy += (mp_limb_t) dy[0]) \
 							name##_set(y__ + jy, x__ + jx); \
 						jy -= ny - 1; \
 					} \
@@ -737,17 +723,17 @@ SEXP R_flint_bits(SEXP object)
 	const void *x = R_flint_get_pointer(object);
 	slong *y = (n) ? flint_calloc(n, sizeof(ulong)) : 0;
 
-#define slong_bits(x) ((slong) FLINT_BIT_COUNT((*(x) < 0) ? (ulong) -(*(x) + 1) + 1 : (ulong) *(x)))
-#define ulong_bits(x) ((slong) FLINT_BIT_COUNT(*(x)))
-#define  fmpq_bits(x) ((slong) fmpq_height_bits(x))
-#define   mag_bits(x) (MAG_MAN(x) ? (slong) (MAG_BITS - flint_ctz(MAG_MAN(x))) : 0)
+#define slong_bits(x) (FLINT_BIT_COUNT((*(x) < 0) ? (ulong) -(*(x) + 1) + 1 : (ulong) *(x)))
+#define ulong_bits(x) (FLINT_BIT_COUNT(*(x)))
+#define  fmpq_bits(x) (fmpq_height_bits(x))
+#define   mag_bits(x) (MAG_MAN(x) ? (MAG_BITS - flint_ctz(MAG_MAN(x))) : 0)
 #define   acf_bits(x) (MAX2(arf_bits(acf_realref(x)), arf_bits(acb_imagref(x))))
 
 #define TEMPLATE(name, elt_t, xptr_t, yptr_t) \
 		do { \
 			xptr_t x__ = x; \
 			for (j = 0; j < n; ++j) \
-				y[j] = name##_bits(x__ + j); \
+				y[j] = (slong) name##_bits(x__ + j); \
 		} while (0)
 
 		R_FLINT_SWITCH(class, TEMPLATE);
@@ -843,8 +829,8 @@ SEXP R_flint_diag(SEXP object, SEXP s_nrow, SEXP s_ncol)
 			         "ncol");
 		int i, j, k = (dy[0] < dy[1]) ? dy[0] : dy[1];
 		if (k > 0) {
-		if ((unsigned int) dy[0] > UWORD_MAX / (unsigned int) dy[1])
-			Rf_error(_("value length would exceed maximum %llu"),
+		if (dy[0] > UWORD_MAX / (mp_limb_t) dy[1])
+			Rf_error(_("length would exceed maximum %llu"),
 			         (unsigned long long int) UWORD_MAX);
 		if (nx == 0)
 			Rf_error(_("'%s' of length zero cannot be recycled to nonzero length"),
@@ -887,7 +873,7 @@ SEXP R_flint_diag(SEXP object, SEXP s_nrow, SEXP s_ncol)
 		SEXP dimx = R_do_slot(object, R_flint_symbol_dim);
 		const int *dx = INTEGER_RO(dimx);
 		int j, k = (dx[0] < dx[1]) ? dx[0] : dx[1];
-		mp_limb_t off = (unsigned int) dx[0] + 1;
+		mp_limb_t off = (mp_limb_t) dx[0] + 1;
 		ny = (mp_limb_t) k;
 
 #define TEMPLATE(name, elt_t, xptr_t, yptr_t) \
@@ -1355,7 +1341,7 @@ SEXP R_flint_rep_each(SEXP object, SEXP s_each, SEXP s_usenames)
 		         "each", "rep");
 	ulong i, each = ((ulong *) R_flint_get_pointer(s_each))[0];
 	if (each > 0 && nx > UWORD_MAX / each)
-		Rf_error(_("value length would exceed maximum %llu"),
+		Rf_error(_("length would exceed maximum %llu"),
 		         (unsigned long long int) UWORD_MAX);
 	ny = nx * each;;
 
@@ -1493,14 +1479,14 @@ SEXP R_flint_rep_times(SEXP object, SEXP s_times, SEXP s_usenames)
 	if (ntimes == 1) {
 		t = times[0];
 		if (t > 0 && nx > UWORD_MAX / t)
-			Rf_error(_("value length would exceed maximum %llu"),
+			Rf_error(_("length would exceed maximum %llu"),
 			         (unsigned long long int) UWORD_MAX);
 		ny = nx * t;
 	} else {
 		for (jx = 0; jx < nx; ++jx) {
 			t = times[jx];
 			if (t > UWORD_MAX - ny)
-				Rf_error(_("value length would exceed maximum %llu"),
+				Rf_error(_("length would exceed maximum %llu"),
 				         (unsigned long long int) UWORD_MAX);
 			ny += t;
 		}
@@ -2144,7 +2130,7 @@ SEXP R_flint_subscript(SEXP object, SEXP subscript, SEXP s_op)
 					Rf_error(_("dimensions would exceed maximum %d"),
 					         INT_MAX);
 				if (t > UWORD_MAX / r)
-					Rf_error(_("value length would exceed maximum %llu"),
+					Rf_error(_("length would exceed maximum %llu"),
 					         (unsigned long long int) UWORD_MAX);
 				if (!anynames && dimnamesx != R_NilValue &&
 				    VECTOR_ELT(dimnamesx, k) != R_NilValue)
@@ -2309,11 +2295,11 @@ SEXP R_flint_transpose(SEXP object, SEXP s_conjugate)
 		what = #name; \
 		if (conjugate) \
 		for (i = 0; i < dx[0]; ++i, jx -= nx - 1) \
-			for (j = 0; j < dx[1]; ++j, jx += dx[0], ++jy) \
+			for (j = 0; j < dx[1]; ++j, jx += (mp_limb_t) dx[0], ++jy) \
 				name##_conj(y__ + jy, x__ + jx); \
 		else \
 		for (i = 0; i < dx[0]; ++i, jx -= nx - 1) \
-			for (j = 0; j < dx[1]; ++j, jx += dx[0], ++jy) \
+			for (j = 0; j < dx[1]; ++j, jx += (mp_limb_t) dx[0], ++jy) \
 				name##_set (y__ + jy, x__ + jx); \
 	} while (0)
 
@@ -2442,7 +2428,7 @@ SEXP R_flint_valid(SEXP object)
 			if (d[i] > UWORD_MAX / l)
 				return INVALID(_("product of '%s' exceeds maximum %llu"),
 				               "dim", UWORD_MAX);
-			l *= (unsigned int) d[i];
+			l *= (mp_limb_t) d[i];
 		}
 		if (l != n)
 			return INVALID(_("product of '%s' [%llu] is not equal to length of object [%llu]"),
@@ -2465,7 +2451,7 @@ SEXP R_flint_valid(SEXP object)
 			if (elt != R_NilValue) {
 				if (TYPEOF(elt) != STRSXP)
 					return INVALID(_("invalid type \"%s\" for %s[[%d]]"),
-					               Rf_type2char(TYPEOF(elt)),
+					               Rf_type2char((SEXPTYPE) TYPEOF(elt)),
 					               "dimnames", (int) i);
 				if (XLENGTH(elt) != d[i])
 					return INVALID(_("length of %s[[%d]] [%lld] is not equal to %s[[%d]] [%lld]"),
