@@ -1,94 +1,8 @@
 #include "flint.h"
 
-#ifndef HAVE_MAG_GET_D_LOWER
-static
-double mag_get_d_lower(const mag_t z)
-{
-	if (mag_is_zero(z))
-		return 0.0;
-	else if (mag_is_inf(z))
-		return D_INF;
-	else if (MAG_EXP(z) < -1000 || MAG_EXP(z) > 1000) {
-		if (fmpz_sgn(MAG_EXPREF(z)) < 0)
-			return 0.0;
-		else
-			return ldexp(1.0, 1000);
-	}
-	else
-		return
-#ifdef HAVE_D_MUL_2EXP
-			d_mul_2exp
-#else
-			ldexp
-#endif
-			((double) MAG_MAN(z), (int) MAG_EXP(z) - MAG_BITS);
-}
-#endif
-
-#ifndef HAVE_MAG_DIV_UI_LOWER
-static R_INLINE
-void mag_div_ui_lower(mag_t z, const mag_t x, ulong y)
-{
-	mag_t t;
-	mag_init(t);
-	mag_set_ui(t, y);
-	mag_div_lower(z, x, t);
-	mag_clear(t);
-	return;
-}
-#endif
-
-#ifndef HAVE_MAG_LOG1P_LOWER
-static R_INLINE
-void mag_log1p_lower(mag_t z, const mag_t x)
-{
-	mag_t t, u;
-	mag_init(t);
-	mag_init(u);
-	mag_one(u);
-	mag_add_lower(t, u, x);
-	mag_log_lower(z, t);
-	mag_clear(t);
-	mag_clear(u);
-	return;
-}
-#endif
-
-#ifndef HAVE_MAG_EXPM1_LOWER
-static R_INLINE
-void mag_expm1_lower(mag_t z, const mag_t x)
-{
-	mag_t t, u;
-	mag_init(t);
-	mag_init(u);
-	mag_one(u);
-	mag_exp_lower(t, x);
-	mag_sub_lower(z, t, u);
-	mag_clear(t);
-	mag_clear(u);
-	return;
-}
-#endif
-
-#define WRAP(op, lower, ...) \
+#define TERN(op, lower, ...) \
 	(lower) ? op##_lower(__VA_ARGS__) : op(__VA_ARGS__)
 
-int isRndZ(mpfr_rnd_t rnd)
-{
-	switch (rnd) {
-	case MPFR_RNDZ:
-	case MPFR_RNDD:
-		return 1;
-	case MPFR_RNDN:
-	case MPFR_RNDU:
-	case MPFR_RNDA:
-		return 0;
-	default:
-		Rf_error(_("should never happen ..."));
-		return -1;
-	}
-}
-
 void R_flint_mag_finalize(SEXP x)
 {
 	mag_ptr p = R_ExternalPtrAddr(x);
@@ -104,11 +18,12 @@ void R_flint_mag_finalize(SEXP x)
 }
 
 SEXP R_flint_mag_initialize(SEXP object, SEXP s_x, SEXP s_length,
-                            SEXP s_dim, SEXP s_dimnames, SEXP s_names)
+                            SEXP s_dim, SEXP s_dimnames, SEXP s_names,
+                            SEXP s_rnd)
 {
-	mp_limb_t jy, nx = 0, ny = 0;
+	mp_limb_t jx, jy, nx = 0, ny = 0;
 	R_flint_class_t class = R_FLINT_CLASS_INVALID;
-	int lower = isRndZ(asRnd(R_NilValue, __func__));
+	int lower = isRndZ(asRnd(s_rnd, 0, __func__));
 	PROTECT(s_dim = validDim(s_dim));
 	PROTECT(s_dimnames = validDimNames(s_dimnames, s_dim));
 	if (s_x != R_NilValue) {
@@ -131,61 +46,64 @@ SEXP R_flint_mag_initialize(SEXP object, SEXP s_x, SEXP s_length,
 	PROTECT(s_names = validNames(s_names, ny));
 	mag_ptr y = (ny) ? flint_calloc(ny, sizeof(mag_t)) : 0;
 	R_flint_set(object, y, ny, (R_CFinalizer_t) &R_flint_mag_finalize);
+	int seenimag = 0, seenrad = 0;
 	switch (TYPEOF(s_x)) {
 	case NILSXP:
-		for (jy = 0; jy < ny; ++jy)
+		FOR_RECYCLE0(jy, ny)
 			mag_zero(y + jy);
 		break;
 	case RAWSXP:
 	{
 		const Rbyte *x = RAW_RO(s_x);
-		for (jy = 0; jy < ny; ++jy)
-			WRAP(mag_set_ui, lower, y + jy, x[jy % nx]);
+		FOR_RECYCLE1(jy, ny, jx, nx)
+			TERN(mag_set_ui, lower, y + jy, x[jx]);
 		break;
 	}
 	case LGLSXP:
 	{
 		const int *x = LOGICAL_RO(s_x);
-		for (jy = 0; jy < ny; ++jy) {
-			if (x[jy % nx] == NA_LOGICAL)
-			Rf_error(_("NaN is not representable by \"%s\""), "mag");
-			else
-			WRAP(mag_set_ui, lower, y + jy, (ulong) x[jy % nx]);
+		FOR_RECYCLE1(jy, ny, jx, nx) {
+			if (x[jx] == NA_LOGICAL)
+				Rf_error(_("NaN is not representable by \"%s\""),
+				         "mag");
+			TERN(mag_set_ui, lower, y + jy, (ulong) (x[jx] != 0));
 		}
 		break;
 	}
 	case INTSXP:
 	{
 		const int *x = INTEGER_RO(s_x);
-		for (jy = 0; jy < ny; ++jy) {
-			if (x[jy % nx] == NA_INTEGER)
-			Rf_error(_("NaN is not representable by \"%s\""), "mag");
-			else if (x[jy % nx] >= 0)
-			WRAP(mag_set_ui, lower, y + jy, (ulong) x[jy % nx]);
+		FOR_RECYCLE1(jy, ny, jx, nx) {
+			if (x[jx] == NA_INTEGER)
+				Rf_error(_("NaN is not representable by \"%s\""),
+				         "mag");
+			if (x[jx] >= 0)
+			TERN(mag_set_ui, lower, y + jy, (ulong)  x[jx]);
 			else
-			WRAP(mag_set_ui, lower, y + jy, (ulong) -x[jy % nx]);
+			TERN(mag_set_ui, lower, y + jy, (ulong) -x[jx]);
 		}
 		break;
 	}
 	case REALSXP:
 	{
 		const double *x = REAL_RO(s_x);
-		for (jy = 0; jy < ny; ++jy) {
-			if (ISNAN(x[jy % nx]))
-			Rf_error(_("NaN is not representable by \"%s\""), "mag");
-			else
-			WRAP(mag_set_d, lower, y + jy, x[jy % nx]);
+		FOR_RECYCLE1(jy, ny, jx, nx) {
+			if (ISNAN(x[jx]))
+				Rf_error(_("NaN is not representable by \"%s\""),
+				         "mag");
+			TERN(mag_set_d, lower, y + jy, x[jx]);
 		}
 		break;
 	}
 	case CPLXSXP:
 	{
 		const Rcomplex *x = COMPLEX_RO(s_x);
-		for (jy = 0; jy < ny; ++jy) {
-			if (ISNAN(x[jy % nx].r))
-			Rf_error(_("NaN is not representable by \"%s\""), "mag");
-			else
-			WRAP(mag_set_d, lower, y + jy, x[jy % nx].r);
+		FOR_RECYCLE1(jy, ny, jx, nx) {
+			if (ISNAN(x[jx].r))
+				Rf_error(_("NaN is not representable by \"%s\""),
+				         "mag");
+			TERN(mag_set_d, lower, y + jy, x[jx].r);
+			seenimag = seenimag || x[jx].i != 0.0;
 		}
 		break;
 	}
@@ -199,8 +117,8 @@ SEXP R_flint_mag_initialize(SEXP object, SEXP s_x, SEXP s_length,
 		arf_init(tmp);
 		const char *s;
 		char *t;
-		for (jy = 0; jy < ny; ++jy) {
-			s = CHAR(STRING_ELT(s_x, (R_xlen_t) (jy % nx)));
+		FOR_RECYCLE1(jy, ny, jx, nx) {
+			s = CHAR(STRING_ELT(s_x, (R_xlen_t) jx));
 			mpfr_strtofr(r, s, &t, 0, rnd);
 			if (t <= s)
 				break;
@@ -210,7 +128,7 @@ SEXP R_flint_mag_initialize(SEXP object, SEXP s_x, SEXP s_length,
 			if (*s != '\0')
 				break;
 			arf_set_mpfr(tmp, r);
-			WRAP(arf_get_mag, lower, y + jy, tmp);
+			TERN(arf_get_mag, lower, y + jy, tmp);
 		}
 		mpfr_clear(r);
 		arf_clear(tmp);
@@ -223,26 +141,26 @@ SEXP R_flint_mag_initialize(SEXP object, SEXP s_x, SEXP s_length,
 		case R_FLINT_CLASS_ULONG:
 		{
 			const ulong *x = R_flint_get_pointer(s_x);
-			for (jy = 0; jy < ny; ++jy)
-				WRAP(mag_set_ui, lower, y + jy, x[jy % nx]);
+			FOR_RECYCLE1(jy, ny, jx, nx)
+				TERN(mag_set_ui, lower, y + jy, x[jx]);
 			break;
 		}
 		case R_FLINT_CLASS_SLONG:
 		{
 			const slong *x = R_flint_get_pointer(s_x);
-			for (jy = 0; jy < ny; ++jy) {
-				if (x[jy % nx] >= 0)
-				WRAP(mag_set_ui, lower, y + jy, (ulong) x[jy % nx]);
+			FOR_RECYCLE1(jy, ny, jx, nx) {
+				if (x[jx] >= 0)
+				TERN(mag_set_ui, lower, y + jy, (ulong) x[jx]);
 				else
-				WRAP(mag_set_ui, lower, y + jy, (ulong) -1 - (ulong) x[jy % nx] + 1);
+				TERN(mag_set_ui, lower, y + jy, (ulong) -1 - (ulong) x[jx] + 1);
 			}
 			break;
 		}
 		case R_FLINT_CLASS_FMPZ:
 		{
 			const fmpz *x = R_flint_get_pointer(s_x);
-			for (jy = 0; jy < ny; ++jy)
-				WRAP(mag_set_fmpz, lower, y + jy, x + jy % nx);
+			FOR_RECYCLE1(jy, ny, jx, nx)
+				TERN(mag_set_fmpz, lower, y + jy, x + jx);
 			break;
 		}
 		case R_FLINT_CLASS_FMPQ:
@@ -252,9 +170,9 @@ SEXP R_flint_mag_initialize(SEXP object, SEXP s_x, SEXP s_length,
 			arf_rnd_t rnd = (lower) ? ARF_RND_DOWN : ARF_RND_UP;
 			arf_t q;
 			arf_init(q);
-			for (jy = 0; jy < ny; ++jy) {
-				arf_fmpz_div_fmpz(q, fmpq_numref(x + jy % nx), fmpq_denref(x + jy % nx), prec, rnd);
-				WRAP(arf_get_mag, lower, y + jy, q);
+			FOR_RECYCLE1(jy, ny, jx, nx) {
+				arf_fmpz_div_fmpz(q, fmpq_numref(x + jx), fmpq_denref(x + jx), prec, rnd);
+				TERN(arf_get_mag, lower, y + jy, q);
 			}
 			arf_clear(q);
 			break;
@@ -262,34 +180,53 @@ SEXP R_flint_mag_initialize(SEXP object, SEXP s_x, SEXP s_length,
 		case R_FLINT_CLASS_MAG:
 		{
 			mag_srcptr x = R_flint_get_pointer(s_x);
-			for (jy = 0; jy < ny; ++jy)
-				mag_set(y + jy, x + jy % nx);
+			FOR_RECYCLE1(jy, ny, jx, nx)
+				mag_set(y + jy, x + jx);
 			break;
 		}
 		case R_FLINT_CLASS_ARF:
 		{
 			arf_srcptr x = R_flint_get_pointer(s_x);
-			for (jy = 0; jy < ny; ++jy)
-				WRAP(arf_get_mag, lower, y + jy, x + jy % nx);
+			FOR_RECYCLE1(jy, ny, jx, nx)
+				TERN(arf_get_mag, lower, y + jy, x + jx);
 			break;
 		}
 		case R_FLINT_CLASS_ACF:
 		{
 			acf_srcptr x = R_flint_get_pointer(s_x);
-			for (jy = 0; jy < ny; ++jy)
-				WRAP(arf_get_mag, lower, y + jy, acf_realref(x + jy % nx));
+			FOR_RECYCLE1(jy, ny, jx, nx) {
+				TERN(arf_get_mag, lower, y + jy, acf_realref(x + jx));
+				seenimag = seenimag || !arf_is_zero(acf_imagref(x + jx));
+			}
 			break;
 		}
 		case R_FLINT_CLASS_ARB:
-		case R_FLINT_CLASS_ACB:
-			Rf_error(_("coercion from ball to point is not yet supported"));
+		{
+			arb_srcptr x = R_flint_get_pointer(s_x);
+			FOR_RECYCLE1(jy, ny, jx, nx) {
+				TERN(arf_get_mag, lower, y + jy, arb_midref(x + jx));
+				seenrad = seenrad || !arb_is_exact(x + jx);
+			}
 			break;
+		}
+		case R_FLINT_CLASS_ACB:
+		{
+			acb_srcptr x = R_flint_get_pointer(s_x);
+			FOR_RECYCLE1(jy, ny, jx, nx) {
+				TERN(arf_get_mag, lower, y + jy, arb_midref(acb_realref(x + jx)));
+				seenimag = seenimag || !arb_is_zero (acb_imagref(x + jx));
+				seenrad  = seenrad  || !arb_is_exact(acb_realref(x + jx));
+			}
+			break;
+		}
 		case R_FLINT_CLASS_INVALID:
 			Rf_error(_("foreign external pointer"));
 			break;
 		}
 		break;
 	}
+	if (seenimag) WARNING_LOST_IMAG;
+	if (seenrad ) WARNING_LOST_RAD ;
 	setDDNN(object, s_dim, s_dimnames, s_names);
 	UNPROTECT(3);
 	return object;
@@ -299,36 +236,37 @@ SEXP R_flint_mag_atomic(SEXP object)
 {
 	mp_limb_t j, n = R_flint_get_length(object);
 	ERROR_TOO_LONG(n, R_XLEN_T_MAX);
-	int lower = isRndZ(asRnd(R_NilValue, __func__));
+	int lower = isRndZ(asRnd(R_NilValue, 0, __func__));
 	SEXP ans = PROTECT(Rf_allocVector(REALSXP, (R_xlen_t) n));
 	mag_srcptr x = R_flint_get_pointer(object);
 	double *y = REAL(ans);
+	int seenoob = 0;
 	mag_t ub;
 	mag_init(ub);
 	mag_set_ui_2exp_si(ub, 1, DBL_MAX_EXP);
-	int w = 1;
 	for (j = 0; j < n; ++j) {
 		if (mag_cmp(x + j, ub) < 0)
-			y[j] = WRAP(mag_get_d, lower, x + j);
+			y[j] = TERN(mag_get_d, lower, x + j);
 		else {
 			y[j] = R_PosInf;
-			WARNING_OOB_DOUBLE(w);
+			seenoob = 1;
 		}
 	}
 	mag_clear(ub);
+	if (seenoob) WARNING_OOB_DOUBLE;
 	UNPROTECT(1);
 	return ans;
 }
 
 SEXP R_flint_mag_format(SEXP object, SEXP s_base,
-                        SEXP s_digits, SEXP s_sep, SEXP s_rnd)
+                        SEXP s_sep, SEXP s_digits, SEXP s_rnd)
 {
 	mp_limb_t j, n = R_flint_get_length(object);
 	ERROR_TOO_LONG(n, R_XLEN_T_MAX);
 	int base = asBase(s_base, __func__), abase = (base < 0) ? -base : base;
 	size_t digits = asDigits(s_digits, __func__);
 	const char *sep = asSep(s_sep, __func__);
-	int lower = isRndZ(asRnd(s_rnd, __func__));
+	int lower = isRndZ(asRnd(s_rnd, 0, __func__));
 	mpfr_rnd_t rnd = (lower) ? MPFR_RNDZ : MPFR_RNDA;
 	SEXP ans = PROTECT(Rf_allocVector(STRSXP, (R_xlen_t) n));
 	if (n) {
@@ -459,8 +397,9 @@ SEXP R_flint_mag_format(SEXP object, SEXP s_base,
 
 SEXP R_flint_mag_ops2(SEXP s_op, SEXP s_x, SEXP s_y, SEXP s_dots)
 {
-	size_t op = strmatch(CHAR(STRING_ELT(s_op, 0)), R_flint_ops2);
-	mp_limb_t jz,
+	R_flint_ops2_t op = ops2match(CHAR(STRING_ELT(s_op, 0)));
+	int info = ops2info(op);
+	mp_limb_t jx, jy, jz,
 		nx = R_flint_get_length(s_x),
 		ny = R_flint_get_length(s_y),
 		nz = RECYCLE2(nx, ny);
@@ -468,46 +407,106 @@ SEXP R_flint_mag_ops2(SEXP s_op, SEXP s_x, SEXP s_y, SEXP s_dots)
 		x = R_flint_get_pointer(s_x),
 		y = R_flint_get_pointer(s_y);
 	int dz[3];
-	int mop = checkConformable(s_x, s_y, nx, ny, matrixop(op), dz);
-	int lower = isRndZ(asRnd(R_NilValue, __func__));
+	info = checkConformable(s_x, s_y, nx, ny, info, dz);
+	int lower = isRndZ(asRnd(R_NilValue, 0, __func__));
 	switch (op) {
-	case  1: /*   "+" */
-	case  2: /*   "-" */
-	case  3: /*   "*" */
-	case  6: /*   "/" */
-	case  7: /*   "^" */
+	case R_FLINT_OPS2_ADD:
+	case R_FLINT_OPS2_SUB:
+	case R_FLINT_OPS2_MUL:
+	case R_FLINT_OPS2_FDR:
+	case R_FLINT_OPS2_FDQ:
+	case R_FLINT_OPS2_DIV:
+	case R_FLINT_OPS2_POW:
 	{
-		SEXP ans = PROTECT(newObject("mag"));
-		mag_ptr z = (nz) ? flint_calloc(nz, sizeof(mag_t)) : 0;
-		R_flint_set(ans, z, nz, (R_CFinalizer_t) &R_flint_mag_finalize);
+		SEXP ans = PROTECT(newFlint(R_FLINT_CLASS_MAG, 0, nz));
+		mag_ptr z = R_flint_get_pointer(ans);
 		switch (op) {
-		case 1: /*   "+" */
-			for (jz = 0; jz < nz; ++jz)
-				WRAP(mag_add, lower, z + jz, x + jz % nx, y + jz % ny);
+		case R_FLINT_OPS2_ADD:
+			FOR_RECYCLE2(jz, nz, jx, nx, jy, ny)
+				TERN(mag_add, lower, z + jz, x + jx, y + jy);
 			break;
-		case 2: /*   "-" */
-			for (jz = 0; jz < nz; ++jz)
-				if (mag_cmp(x + jz % nx, y + jz % ny) >= 0)
-				WRAP(mag_sub, lower, z + jz, x + jz % nx, y + jz % ny);
+		case R_FLINT_OPS2_SUB:
+			FOR_RECYCLE2(jz, nz, jx, nx, jy, ny)
+				if (mag_cmp(x + jx, y + jy) >= 0)
+				TERN(mag_sub, lower, z + jz, x + jx, y + jy);
 				else
-				WRAP(mag_sub, lower, z + jz, y + jz % ny, x + jz % nx);
+				TERN(mag_sub, lower, z + jz, y + jy, x + jx);
 			break;
-		case 3: /*   "*" */
-			for (jz = 0; jz < nz; ++jz)
-				WRAP(mag_mul, lower, z + jz, x + jz % nx, y + jz % ny);
+		case R_FLINT_OPS2_MUL:
+			FOR_RECYCLE2(jz, nz, jx, nx, jy, ny)
+				TERN(mag_mul, lower, z + jz, x + jx, y + jy);
 			break;
-		case 6: /*   "/" */
-			for (jz = 0; jz < nz; ++jz)
-				WRAP(mag_div, lower, z + jz, x + jz % nx, y + jz % ny);
+		case R_FLINT_OPS2_FDR:
+		{
+			int cmp;
+			mag_t q, t;
+			fmpz_t f;
+			mag_init(q);
+			mag_init(t);
+			fmpz_init(f);
+			FOR_RECYCLE2(jz, nz, jx, nx, jy, ny) {
+				if (mag_is_zero(y + jy))
+					Rf_error(_("NaN is not representable by \"%s\""), "mag");
+				cmp = mag_cmp(x + jx, y + jy);
+				if (cmp == 0)
+					mag_zero(z + jz);
+				else if (cmp < 0)
+					mag_set(z + jz, x + jx);
+				else {
+					mag_div_lower(q, x + jx, y + jy);
+					mag_get_fmpz_lower(f, q);
+					mag_mul_fmpz_lower(t, q, f);
+					mag_sub(z + jz, x + jx, t);
+				}
+			}
+			mag_clear(q);
+			mag_clear(t);
+			fmpz_clear(f);
 			break;
-		case 7: /*   "^" */
+		}
+		case R_FLINT_OPS2_FDQ:
+		{
+			int cmp;
+			mag_t q;
+			fmpz_t f;
+			mag_init(q);
+			fmpz_init(f);
+			FOR_RECYCLE2(jz, nz, jx, nx, jy, ny) {
+				if (mag_is_zero(y + jy))
+					Rf_error(_("NaN is not representable by \"%s\""), "mag");
+				cmp = mag_cmp(x + jx, y + jy);
+				if (cmp == 0)
+					mag_one(z + jz);
+				else if (cmp < 0)
+					mag_zero(z + jz);
+				else {
+					mag_div(q, x + jx, y + jy);
+					mag_get_fmpz_lower(f, q);
+					mag_set_fmpz(z + jz, f);
+				}
+			}
+			mag_clear(q);
+			fmpz_clear(f);
+			break;
+		}
+		case R_FLINT_OPS2_DIV:
+			FOR_RECYCLE2(jz, nz, jx, nx, jy, ny) {
+				if (mag_is_special(x + jx) &&
+				    mag_is_special(y + jy) &&
+				    (mag_is_zero(x + jx) != 0) ==
+				    (mag_is_zero(y + jy) != 0))
+					Rf_error(_("NaN is not representable by \"%s\""), "mag");
+				TERN(mag_div, lower, z + jz, x + jx, y + jy);
+			}
+			break;
+		case R_FLINT_OPS2_POW:
 		{
 			mag_srcptr b, e;
 			mag_t a;
 			mag_init(a);
-			for (jz = 0; jz < nz; ++jz) {
-				b = x + jz % nx;
-				e = y + jz % ny;
+			FOR_RECYCLE2(jz, nz, jx, nx, jy, ny) {
+				b = x + jx;
+				e = y + jy;
 				if (mag_is_zero(e) || mag_cmp_2exp_si(b, 0) == 0)
 					/* b^0, 1^e = 1 */
 					mag_one(z + jz);
@@ -523,72 +522,74 @@ SEXP R_flint_mag_ops2(SEXP s_op, SEXP s_x, SEXP s_y, SEXP s_dots)
 				}
 				else if (mag_cmp_2exp_si(b, 0) > 0) {
 					/* b^e = exp(e * log(b)) */
-					WRAP(mag_log, lower, a, b);
-					WRAP(mag_mul, lower, a, e, a);
-					WRAP(mag_exp, lower, z + jz, a);
+					TERN(mag_log, lower, a, b);
+					TERN(mag_mul, lower, a, e, a);
+					TERN(mag_exp, lower, z + jz, a);
 				}
 				else {
 					/* b^e = exp(-(e * -log(b))) */
-					WRAP(mag_neg_log, !lower, a, b);
-					WRAP(mag_mul, !lower, a, e, a);
-					WRAP(mag_expinv, lower, z + jz, a);
+					TERN(mag_neg_log, !lower, a, b);
+					TERN(mag_mul, !lower, a, e, a);
+					TERN(mag_expinv, lower, z + jz, a);
 				}
 			}
 			mag_clear(a);
 			break;
 		}
+		default: /* -Wswitch */
 		}
-		setDDNN2(ans, s_x, s_y, nz, nx, ny, mop);
+		setDDNN2(ans, s_x, s_y, nz, nx, ny, info);
 		UNPROTECT(1);
 		return ans;
 	}
-	case  8: /*  "==" */
-	case  9: /*  "!=" */
-	case 10: /*   "<" */
-	case 11: /*   ">" */
-	case 12: /*  "<=" */
-	case 13: /*  ">=" */
-	case 14: /*   "&" */
-	case 15: /*   "|" */
+	case R_FLINT_OPS2_EQ:
+	case R_FLINT_OPS2_NEQ:
+	case R_FLINT_OPS2_L:
+	case R_FLINT_OPS2_G:
+	case R_FLINT_OPS2_LEQ:
+	case R_FLINT_OPS2_GEQ:
+	case R_FLINT_OPS2_AND:
+	case R_FLINT_OPS2_OR:
 	{
 		ERROR_TOO_LONG(nz, R_XLEN_T_MAX);
 		SEXP ans = PROTECT(Rf_allocVector(LGLSXP, (R_xlen_t) nz));
 		int *z = LOGICAL(ans);
 		switch (op) {
-		case  8: /*  "==" */
-			for (jz = 0; jz < nz; ++jz)
-				z[jz] = mag_equal(x + jz % nx, y + jz % ny) != 0;
+		case R_FLINT_OPS2_EQ:
+			FOR_RECYCLE2(jz, nz, jx, nx, jy, ny)
+				z[jz] = mag_equal(x + jx, y + jy) != 0;
 			break;
-		case  9: /*  "!=" */
-			for (jz = 0; jz < nz; ++jz)
-				z[jz] = mag_equal(x + jz % nx, y + jz % ny) == 0;
+		case R_FLINT_OPS2_NEQ:
+			FOR_RECYCLE2(jz, nz, jx, nx, jy, ny)
+				z[jz] = mag_equal(x + jx, y + jy) == 0;
 			break;
-		case 10: /*   "<" */
-			for (jz = 0; jz < nz; ++jz)
-				z[jz] = mag_cmp(x + jz % nx, y + jz % ny) < 0;
+		case R_FLINT_OPS2_L:
+			FOR_RECYCLE2(jz, nz, jx, nx, jy, ny)
+				z[jz] = mag_cmp(x + jx, y + jy) < 0;
 			break;
-		case 11: /*   ">" */
-			for (jz = 0; jz < nz; ++jz)
-				z[jz] = mag_cmp(x + jz % nx, y + jz % ny) > 0;
+		case R_FLINT_OPS2_G:
+			FOR_RECYCLE2(jz, nz, jx, nx, jy, ny)
+				z[jz] = mag_cmp(x + jx, y + jy) > 0;
 			break;
-		case 12: /*  "<=" */
-			for (jz = 0; jz < nz; ++jz)
-				z[jz] = mag_cmp(x + jz % nx, y + jz % ny) <= 0;
+		case R_FLINT_OPS2_LEQ:
+			FOR_RECYCLE2(jz, nz, jx, nx, jy, ny)
+				z[jz] = mag_cmp(x + jx, y + jy) <= 0;
 			break;
-		case 13: /*  ">=" */
-			for (jz = 0; jz < nz; ++jz)
-				z[jz] = mag_cmp(x + jz % nx, y + jz % ny) >= 0;
+		case R_FLINT_OPS2_GEQ:
+			FOR_RECYCLE2(jz, nz, jx, nx, jy, ny)
+				z[jz] = mag_cmp(x + jx, y + jy) >= 0;
 			break;
-		case 14: /*   "&" */
-			for (jz = 0; jz < nz; ++jz)
-				z[jz] = !mag_is_zero(x + jz % nx) && !mag_is_zero(y + jz % ny);
+		case R_FLINT_OPS2_AND:
+			FOR_RECYCLE2(jz, nz, jx, nx, jy, ny)
+				z[jz] = !mag_is_zero(x + jx) && !mag_is_zero(y + jy);
 			break;
-		case 15: /*   "|" */
-			for (jz = 0; jz < nz; ++jz)
-				z[jz] = !mag_is_zero(x + jz % nx) || !mag_is_zero(y + jz % ny);
+		case R_FLINT_OPS2_OR:
+			FOR_RECYCLE2(jz, nz, jx, nx, jy, ny)
+				z[jz] = !mag_is_zero(x + jx) || !mag_is_zero(y + jy);
 			break;
+		default: /* -Wswitch */
 		}
-		setDDNN2(ans, s_x, s_y, nz, nx, ny, mop);
+		setDDNN2(ans, s_x, s_y, nz, nx, ny, info);
 		UNPROTECT(1);
 		return ans;
 	}
@@ -601,74 +602,68 @@ SEXP R_flint_mag_ops2(SEXP s_op, SEXP s_x, SEXP s_y, SEXP s_dots)
 
 SEXP R_flint_mag_ops1(SEXP s_op, SEXP s_x, SEXP s_dots)
 {
-	size_t op = strmatch(CHAR(STRING_ELT(s_op, 0)), R_flint_ops1);
+	R_flint_ops1_t op = ops1match(CHAR(STRING_ELT(s_op, 0)));
+	int info = ops1info(op);
 	mp_limb_t jx, jz, nx = R_flint_get_length(s_x), nz = nx;
 	mag_srcptr x = R_flint_get_pointer(s_x);
-	int lower = isRndZ(asRnd(R_NilValue, __func__));
+	int lower = isRndZ(asRnd(R_NilValue, 0, __func__));
 	switch (op) {
-	case  1: /*       "+" */
-	case  2: /*       "-" */
-	case  8: /*    "Conj" */
-	case  9: /*      "Re" */
-	case 10: /*      "Im" */
-	case 11: /*     "Mod" */
-	case 12: /*     "Arg" */
-	case 13: /*     "abs" */
-	case 14: /*    "sign" */
-	case 15: /*    "sqrt" */
-	case 16: /*   "floor" */
-	case 17: /* "ceiling" */
-	case 18: /*   "trunc" */
-	case 19: /*  "cummin" */
-	case 20: /*  "cummax" */
-	case 21: /*  "cumsum" */
-	case 22: /* "cumprod" */
-	case 23: /*     "log" */
-	case 24: /*   "log10" */
-	case 25: /*    "log2" */
-	case 26: /*   "log1p" */
-	case 27: /*     "exp" */
-	case 28: /*   "expm1" */
-	case 48: /*   "round" */
-	case 49: /*  "signif" */
+	case R_FLINT_OPS1_PLUS:
+	case R_FLINT_OPS1_MINUS:
+	case R_FLINT_OPS1_CONJ:
+	case R_FLINT_OPS1_REAL:
+	case R_FLINT_OPS1_IMAG:
+	case R_FLINT_OPS1_MOD:
+	case R_FLINT_OPS1_ARG:
+	case R_FLINT_OPS1_ABS:
+	case R_FLINT_OPS1_SIGN:
+	case R_FLINT_OPS1_SQRT:
+	case R_FLINT_OPS1_FLOOR:
+	case R_FLINT_OPS1_CEILING:
+	case R_FLINT_OPS1_TRUNC:
+	case R_FLINT_OPS1_CUMMIN:
+	case R_FLINT_OPS1_CUMMAX:
+	case R_FLINT_OPS1_CUMSUM:
+	case R_FLINT_OPS1_CUMPROD:
+	case R_FLINT_OPS1_LOG:
+	case R_FLINT_OPS1_LOG10:
+	case R_FLINT_OPS1_LOG2:
+	case R_FLINT_OPS1_LOG1P:
+	case R_FLINT_OPS1_EXP:
+	case R_FLINT_OPS1_EXPM1:
+	case R_FLINT_OPS1_ROUND:
+	case R_FLINT_OPS1_SIGNIF:
 	{
-		SEXP ans = PROTECT(newObject("mag"));
-		mag_ptr z = (nz) ? flint_calloc(nz, sizeof(mag_t)) : 0;
-		R_flint_set(ans, z, nz, (R_CFinalizer_t) &R_flint_mag_finalize);
+		SEXP ans = PROTECT(newFlint(R_FLINT_CLASS_MAG, 0, nz));
+		mag_ptr z = R_flint_get_pointer(ans);
 		switch (op) {
-		case  1: /*       "+" */
-		case  2: /*       "-" */
-		case  8: /*    "Conj" */
-		case  9: /*      "Re" */
-		case 11: /*     "Mod" */
-		case 13: /*     "abs" */
+		case R_FLINT_OPS1_PLUS:
+		case R_FLINT_OPS1_MINUS:
+		case R_FLINT_OPS1_CONJ:
+		case R_FLINT_OPS1_REAL:
+		case R_FLINT_OPS1_MOD:
+		case R_FLINT_OPS1_ABS:
 			for (jz = 0; jz < nz; ++jz)
 				mag_set(z + jz, x + jz);
 			break;
-		case 10: /*      "Im" */
+		case R_FLINT_OPS1_IMAG:
+		case R_FLINT_OPS1_ARG:
 			for (jz = 0; jz < nz; ++jz)
 				mag_zero(z + jz);
 			break;
-		case 12: /*     "Arg" */
-			for (jz = 0; jz < nz; ++jz)
-				if (mag_is_zero(x + jz))
-					mag_zero(z + jz);
-				else
-					WRAP(mag_const_pi, lower, z + jz);
-			break;
-		case 14: /*    "sign" */
+		case R_FLINT_OPS1_SIGN:
 			for (jz = 0; jz < nz; ++jz)
 				if (mag_is_zero(x + jz))
 					mag_zero(z + jz);
 				else
 					mag_one(z + jz);
 			break;
-		case 15: /*    "sqrt" */
+		case R_FLINT_OPS1_SQRT:
 			for (jz = 0; jz < nz; ++jz)
-				WRAP(mag_sqrt, lower, z + jz, x + jz);
+				TERN(mag_sqrt, lower, z + jz, x + jz);
 			break;
-		case 16: /*   "floor" */
-		case 18: /*   "trunc" */
+		case R_FLINT_OPS1_FLOOR:
+		case R_FLINT_OPS1_TRUNC:
 		{
 			fmpz_t r;
 			fmpz_init(r);
@@ -683,13 +678,13 @@ SEXP R_flint_mag_ops1(SEXP s_op, SEXP s_x, SEXP s_dots)
 					mag_set(z + jz, x + jz);
 				else {
 					mag_get_fmpz_lower(r, x + jz);
-					WRAP(mag_set_fmpz, lower, z + jz, r);
+					TERN(mag_set_fmpz, lower, z + jz, r);
 				}
 			}
 			fmpz_clear(r);
 			break;
 		}
-		case 17: /* "ceiling" */
+		case R_FLINT_OPS1_CEILING:
 		{
 			fmpz_t r;
 			fmpz_init(r);
@@ -704,110 +699,115 @@ SEXP R_flint_mag_ops1(SEXP s_op, SEXP s_x, SEXP s_dots)
 					mag_set(z + jz, x + jz);
 				else {
 					mag_get_fmpz(r, x + jz);
-					WRAP(mag_set_fmpz, lower, z + jz, r);
+					TERN(mag_set_fmpz, lower, z + jz, r);
 				}
 			}
 			fmpz_clear(r);
 			break;
 		}
-		case 19: /*  "cummin" */
+		case R_FLINT_OPS1_CUMMIN:
 			if (nz) {
 			mag_set(z, x);
 			for (jz = 1; jz < nz; ++jz)
 				mag_min(z + jz, z + jz - 1, x + jz);
 			}
 			break;
-		case 20: /*  "cummax" */
+		case R_FLINT_OPS1_CUMMAX:
 			if (nz) {
 			mag_set(z, x);
 			for (jz = 1; jz < nz; ++jz)
 				mag_max(z + jz, z + jz - 1, x + jz);
 			}
 			break;
-		case 21: /*  "cumsum" */
+		case R_FLINT_OPS1_CUMSUM:
 			if (nz) {
 			mag_set(z, x);
 			for (jz = 1; jz < nz; ++jz)
-				WRAP(mag_add, lower, z + jz, z + jz - 1, x + jz);
+				TERN(mag_add, lower, z + jz, z + jz - 1, x + jz);
 			}
 			break;
-		case 22: /* "cumprod" */
+		case R_FLINT_OPS1_CUMPROD:
 			if (nz) {
 			mag_set(z, x);
 			for (jz = 1; jz < nz; ++jz)
-				WRAP(mag_mul, lower, z + jz, z + jz - 1, x + jz);
+				TERN(mag_mul, lower, z + jz, z + jz - 1, x + jz);
 			}
 			break;
-		case 23: /*     "log" */
-		case 24: /*   "log10" */
-		case 25: /*    "log2" */
-			for (jz = 0; jz < nz; ++jz)
+		case R_FLINT_OPS1_LOG:
+			if (s_dots == R_NilValue)
+			for (jz = 0; jz < nz; ++jz) {
 				if (mag_cmp_2exp_si(x + jz, 0) >= 0)
-					WRAP(mag_log, lower, z + jz, x + jz);
+					TERN(mag_log    , lower, z + jz, x + jz);
 				else
-					WRAP(mag_neg_log, lower, z + jz, x + jz);
-			if (op != 23 || s_dots != R_NilValue) {
-			mag_t tmp;
-			mag_init(tmp);
-			if (op != 23)
-				WRAP(mag_set_ui, !lower, tmp, (op == 24) ? 10 : 2);
-			else {
-				SEXP s_base = VECTOR_ELT(s_dots, 0);
-				if (R_flint_get_length(s_base) == 0)
-					Rf_error(_("'%s' of length zero in '%s'"),
-					         "base", CHAR(STRING_ELT(s_op, 0)));
-				arf_srcptr base = R_flint_get_pointer(s_base);
-				if (arf_is_nan(base) || arf_sgn(base) < 0) {
-					mag_clear(tmp);
-					Rf_error(_("NaN is not representable by \"%s\""), "mag");
-				}
-				if (arf_cmp_2exp_si(base, 0) >= 0)
-					WRAP(arf_get_mag, !lower, tmp, base);
-				else
-					WRAP(arf_get_mag, lower, tmp, base);
-			}
-			if (mag_cmp_2exp_si(tmp, 0) >= 0)
-				WRAP(mag_log, !lower, tmp, tmp);
-			else
-				WRAP(mag_neg_log, !lower, tmp, tmp);
-			if (mag_is_special(tmp)) {
-				for (jz = 0; jz < nz; ++jz)
-					if (mag_is_inf(z + jz)) {
-					mag_clear(tmp);
-					Rf_error(_("NaN is not representable by \"%s\""), "mag");
-					}
-					else
-					mag_zero(z + jz);
-			}
-			else if (mag_cmp_2exp_si(tmp, 0) == 0) {
-				for (jz = 0; jz < nz; ++jz)
-					if (mag_is_zero(z + jz)) {
-					mag_clear(tmp);
-					Rf_error(_("NaN is not representable by \"%s\""), "mag");
-					}
-					else
-					mag_inf(z + jz);
+					TERN(mag_neg_log, lower, z + jz, x + jz);
 			}
 			else {
-				for (jz = 0; jz < nz; ++jz)
-					WRAP(mag_div, lower, z + jz, z + jz, tmp);
+			SEXP s_base = VECTOR_ELT(s_dots, 0);
+			if (R_flint_get_length(s_base) == 0)
+				Rf_error(_("'%s' of length zero in '%s'"),
+				         "base", CHAR(STRING_ELT(s_op, 0)));
+			arf_srcptr base = R_flint_get_pointer(s_base);
+			int status = arf_is_nan(base) || arf_sgn(base) < 0;
+			if (status)
+				Rf_error(_("NaN is not representable by \"%s\""), "mag");
+			mag_t t;
+			mag_init(t);
+			if (arf_cmp_2exp_si(base, 0) >= 0) {
+				TERN(arf_get_mag, !lower, t, base);
+				TERN(mag_log    , !lower, t, t);
+			} else {
+				TERN(arf_get_mag,  lower, t, base);
+				TERN(mag_neg_log, !lower, t, t);
 			}
-			mag_clear(tmp);
+			for (jz = 0; jz < nz && !status; ++jz) {
+				if (mag_cmp_2exp_si(x + jz, 0) >= 0)
+					TERN(mag_log    , lower, z + jz, x + jz);
+				else
+					TERN(mag_neg_log, lower, z + jz, x + jz);
+				status =
+					mag_is_special(z + jz) &&
+					mag_is_special(t     ) &&
+					(mag_is_zero(z + jz) != 0) ==
+					(mag_is_zero(t     ) != 0);
+				if (!status)
+				TERN(mag_div, lower, z + jz, z + jz, t);
+			}
+			mag_clear(t);
+			if (status)
+				Rf_error(_("NaN is not representable by \"%s\""), "mag");
 			}
 			break;
-		case 26: /*   "log1p" */
-			for (jz = 0; jz < nz; ++jz)
-				WRAP(mag_log1p, lower, z + jz, x + jz);
+		case R_FLINT_OPS1_LOG10:
+		case R_FLINT_OPS1_LOG2:
+		{
+			ulong b = (op == R_FLINT_OPS1_LOG10) ? 10 : 2;
+			mag_t t;
+			mag_init(t);
+			TERN(mag_set_ui, !lower, t, b);
+			TERN(mag_log   , !lower, t, t);
+			for (jz = 0; jz < nz; ++jz) {
+				if (mag_cmp_2exp_si(x + jz, 0) >= 0)
+					TERN(mag_log    , lower, z + jz, x + jz);
+				else
+					TERN(mag_neg_log, lower, z + jz, x + jz);
+				TERN(mag_div, lower, z + jz, z + jz, t);
+			}
+			mag_clear(t);
 			break;
-		case 27: /*     "exp" */
+		}
+		case R_FLINT_OPS1_LOG1P:
 			for (jz = 0; jz < nz; ++jz)
-				WRAP(mag_exp, lower, z + jz, x + jz);
+				TERN(mag_log1p, lower, z + jz, x + jz);
 			break;
-		case 28: /*   "expm1" */
+		case R_FLINT_OPS1_EXP:
 			for (jz = 0; jz < nz; ++jz)
-				WRAP(mag_expm1, lower, z + jz, x + jz);
+				TERN(mag_exp, lower, z + jz, x + jz);
 			break;
-		case 48: /*   "round" */
+		case R_FLINT_OPS1_EXPM1:
+			for (jz = 0; jz < nz; ++jz)
+				TERN(mag_expm1, lower, z + jz, x + jz);
+			break;
+		case R_FLINT_OPS1_ROUND:
 		{
 			SEXP s_digits = VECTOR_ELT(s_dots, 0);
 			if (R_flint_get_length(s_digits) == 0)
@@ -833,7 +833,7 @@ SEXP R_flint_mag_ops1(SEXP s_op, SEXP s_x, SEXP s_dots)
 				arf_mul_fmpz(s, s, p, ARF_PREC_EXACT, rnd);
 				arf_get_fmpz(q, s, ARF_RND_NEAR);
 				arf_fmpz_div_fmpz(s, q, p, prec, rnd);
-				WRAP(arf_get_mag, lower, z + jz, s);
+				TERN(arf_get_mag, lower, z + jz, s);
 				}
 			}
 			} else {
@@ -847,7 +847,7 @@ SEXP R_flint_mag_ops1(SEXP s_op, SEXP s_x, SEXP s_dots)
 				arf_div_fmpz(s, s, p, prec, rnd);
 				arf_get_fmpz(q, s, ARF_RND_NEAR);
 				fmpz_mul(q, q, p);
-				WRAP(mag_set_fmpz, lower, z + jz, q);
+				TERN(mag_set_fmpz, lower, z + jz, q);
 				}
 			}
 			}
@@ -856,7 +856,7 @@ SEXP R_flint_mag_ops1(SEXP s_op, SEXP s_x, SEXP s_dots)
 			arf_clear(s);
 			break;
 		}
-		case 49: /*  "signif" */
+		case R_FLINT_OPS1_SIGNIF:
 		{
 			slong fmpq_clog_ui(const fmpq_t, ulong);
 			SEXP s_digits = VECTOR_ELT(s_dots, 0);
@@ -895,7 +895,7 @@ SEXP R_flint_mag_ops1(SEXP s_op, SEXP s_x, SEXP s_dots)
 				    fmpz_is_odd(q))
 					fmpz_add_si(q, q, fmpz_sgn(r));
 				arf_fmpz_div_fmpz(s, q, p, prec, rnd);
-				WRAP(arf_get_mag, lower, z + jz, s);
+				TERN(arf_get_mag, lower, z + jz, s);
 				} else {
 				fmpz_pow_ui(p, p, (ulong) (clog - digits));
 				fmpz_mul(fmpq_denref(a), fmpq_denref(a), p);
@@ -904,7 +904,7 @@ SEXP R_flint_mag_ops1(SEXP s_op, SEXP s_x, SEXP s_dots)
 				    fmpz_is_odd(q))
 					fmpz_add_si(q, q, fmpz_sgn(r));
 				fmpz_mul(q, q, p);
-				WRAP(mag_set_fmpz, lower, z + jz, q);
+				TERN(mag_set_fmpz, lower, z + jz, q);
 				}
 				}
 			}
@@ -915,39 +915,173 @@ SEXP R_flint_mag_ops1(SEXP s_op, SEXP s_x, SEXP s_dots)
 			arf_clear(s);
 			break;
 		}
+		default: /* -Wswitch */
 		}
 		setDDNN1(ans, s_x);
 		UNPROTECT(1);
 		return ans;
 	}
-	case 55: /*    "mean" */
+	case R_FLINT_OPS1_COS:
+	case R_FLINT_OPS1_COSPI:
+	case R_FLINT_OPS1_ACOS:
+	case R_FLINT_OPS1_COSH:
+	case R_FLINT_OPS1_ACOSH:
+	case R_FLINT_OPS1_SIN:
+	case R_FLINT_OPS1_SINPI:
+	case R_FLINT_OPS1_ASIN:
+	case R_FLINT_OPS1_SINH:
+	case R_FLINT_OPS1_ASINH:
+	case R_FLINT_OPS1_TAN:
+	case R_FLINT_OPS1_TANPI:
+	case R_FLINT_OPS1_ATAN:
+	case R_FLINT_OPS1_TANH:
+	case R_FLINT_OPS1_ATANH:
+	case R_FLINT_OPS1_GAMMA:
+	case R_FLINT_OPS1_LGAMMA:
+	case R_FLINT_OPS1_2GAMMA:
+	case R_FLINT_OPS1_3GAMMA:
+	{
+		SEXP ans = PROTECT(newFlint(R_FLINT_CLASS_MAG, 0, nz));
+		mag_ptr z = R_flint_get_pointer(ans);
+		int status = 0;
+		arb_t zb, xb;
+		arb_init(zb);
+		arb_init(xb);
+
+#define WRAP(op, z, x) \
+		do { \
+			arf_set_mag(arb_midref(xb), x); \
+			op(zb, xb, MAG_BITS << 1); \
+			status = arf_is_nan(arb_midref(zb)); \
+			if (!status) \
+			TERN(arf_get_mag, lower, z, arb_midref(zb)); \
+		} while (0)
+
+		switch (op) {
+		case R_FLINT_OPS1_COS:
+			for (jz = 0; jz < nz && !status; ++jz)
+				WRAP(arb_cos, z + jz, x + jz);
+			break;
+		case R_FLINT_OPS1_COSPI:
+			for (jz = 0; jz < nz && !status; ++jz)
+				WRAP(arb_cos_pi, z + jz, x + jz);
+			break;
+		case R_FLINT_OPS1_ACOS:
+			for (jz = 0; jz < nz && !status; ++jz)
+				WRAP(arb_acos, z + jz, x + jz);
+			break;
+		case R_FLINT_OPS1_COSH:
+			for (jz = 0; jz < nz && !status; ++jz)
+				WRAP(arb_cosh, z + jz, x + jz);
+			break;
+		case R_FLINT_OPS1_ACOSH:
+			for (jz = 0; jz < nz && !status; ++jz)
+				WRAP(arb_acosh, z + jz, x + jz);
+			break;
+		case R_FLINT_OPS1_SIN:
+			for (jz = 0; jz < nz && !status; ++jz)
+				WRAP(arb_sin, z + jz, x + jz);
+			break;
+		case R_FLINT_OPS1_SINPI:
+			for (jz = 0; jz < nz && !status; ++jz)
+				WRAP(arb_sin_pi, z + jz, x + jz);
+			break;
+		case R_FLINT_OPS1_ASIN:
+			for (jz = 0; jz < nz && !status; ++jz)
+				WRAP(arb_asin, z + jz, x + jz);
+			break;
+		case R_FLINT_OPS1_SINH:
+			for (jz = 0; jz < nz && !status; ++jz)
+				WRAP(arb_sinh, z + jz, x + jz);
+			break;
+		case R_FLINT_OPS1_ASINH:
+			for (jz = 0; jz < nz && !status; ++jz)
+				WRAP(arb_asinh, z + jz, x + jz);
+			break;
+		case R_FLINT_OPS1_TAN:
+			for (jz = 0; jz < nz && !status; ++jz)
+				WRAP(arb_tan, z + jz, x + jz);
+			break;
+		case R_FLINT_OPS1_TANPI:
+			for (jz = 0; jz < nz && !status; ++jz)
+				WRAP(arb_tan_pi, z + jz, x + jz);
+			break;
+		case R_FLINT_OPS1_ATAN:
+			for (jz = 0; jz < nz && !status; ++jz)
+				WRAP(arb_atan, z + jz, x + jz);
+			break;
+		case R_FLINT_OPS1_TANH:
+			for (jz = 0; jz < nz && !status; ++jz)
+				WRAP(arb_tanh, z + jz, x + jz);
+			break;
+		case R_FLINT_OPS1_ATANH:
+			for (jz = 0; jz < nz && !status; ++jz)
+				WRAP(arb_atanh, z + jz, x + jz);
+			break;
+		case R_FLINT_OPS1_GAMMA:
+			for (jz = 0; jz < nz && !status; ++jz)
+				WRAP(arb_gamma, z + jz, x + jz);
+			break;
+		case R_FLINT_OPS1_LGAMMA:
+			for (jz = 0; jz < nz && !status; ++jz)
+				WRAP(arb_lgamma, z + jz, x + jz);
+			break;
+		case R_FLINT_OPS1_2GAMMA:
+			for (jz = 0; jz < nz && !status; ++jz)
+				WRAP(arb_digamma, z + jz, x + jz);
+			break;
+		case R_FLINT_OPS1_3GAMMA:
+		{
+			arb_t s;
+			arb_init(s);
+			arb_set_si(s, 1);
+#define arb_trigamma(z, x, prec) arb_polygamma(z, s, x, prec)
+			for (jz = 0; jz < nz && !status; ++jz)
+				WRAP(arb_trigamma, z + jz, x + jz);
+#undef arb_trigamma
+			arb_clear(s);
+			break;
+		}
+		default: /* -Wswitch */
+		}
+
+#undef WRAP
+
+		arb_clear(zb);
+		arb_clear(xb);
+		if (status)
+			Rf_error(_("NaN is not representable by \"%s\""), "mag");
+		setDDNN1(ans, s_x);
+		UNPROTECT(1);
+		return ans;
+	}
+	case R_FLINT_OPS1_MEAN:
 		if (nx == 0)
 			Rf_error(_("'%s' of length zero in '%s'"),
 			         "x", CHAR(STRING_ELT(s_op, 0)));
-	case 50: /*     "min" */
-	case 51: /*     "max" */
-	case 52: /*   "range" */
-	case 53: /*     "sum" */
-	case 54: /*    "prod" */
+	case R_FLINT_OPS1_MIN:
+	case R_FLINT_OPS1_MAX:
+	case R_FLINT_OPS1_RANGE:
+	case R_FLINT_OPS1_SUM:
+	case R_FLINT_OPS1_PROD:
 	{
-		nz = (op == 52) ? 2 : 1;
-		SEXP ans = PROTECT(newObject("mag"));
-		mag_ptr z = flint_calloc(nz, sizeof(mag_t));
-		R_flint_set(ans, z, nz, (R_CFinalizer_t) &R_flint_mag_finalize);
+		nz = (op == R_FLINT_OPS1_RANGE) ? 2 : 1;
+		SEXP ans = PROTECT(newFlint(R_FLINT_CLASS_MAG, 0, nz));
+		mag_ptr z = R_flint_get_pointer(ans);
 		switch (op) {
-		case 50: /*     "min" */
+		case R_FLINT_OPS1_MIN:
 			mag_inf(z);
 			for (jx = 0; jx < nx; ++jx)
 				if (mag_cmp(z, x + jx) > 0)
 					mag_set(z, x + jx);
 			break;
-		case 51: /*     "max" */
+		case R_FLINT_OPS1_MAX:
 			mag_zero(z);
 			for (jx = 0; jx < nx; ++jx)
 				if (mag_cmp(z, x + jx) < 0)
 					mag_set(z, x + jx);
 			break;
-		case 52: /*   "range" */
+		case R_FLINT_OPS1_RANGE:
 		{
 			mag_inf(z);
 #ifdef R_FLINT_USE_NAIVE_RANGE
@@ -972,48 +1106,47 @@ SEXP R_flint_mag_ops1(SEXP s_op, SEXP s_x, SEXP s_dots)
 #endif
 			break;
 		}
-		case 53: /*     "sum" */
+		case R_FLINT_OPS1_SUM:
 			mag_zero(z);
 			for (jx = 0; jx < nx; ++jx)
-				WRAP(mag_add, lower, z, z, x + jx);
+				TERN(mag_add, lower, z, z, x + jx);
 			break;
-		case 54: /*    "prod" */
+		case R_FLINT_OPS1_PROD:
 			mag_one(z);
 			for (jx = 0; jx < nx; ++jx)
-				WRAP(mag_mul, lower, z, z, x + jx);
+				TERN(mag_mul, lower, z, z, x + jx);
 			break;
-		case 55: /*    "mean" */
-		{
+		case R_FLINT_OPS1_MEAN:
 			mag_zero(z);
 			for (jx = 0; jx < nx; ++jx)
-				WRAP(mag_add, lower, z, z, x + jx);
-			WRAP(mag_div_ui, lower, z, z, nx);
+				TERN(mag_add, lower, z, z, x + jx);
+			TERN(mag_div_ui, lower, z, z, nx);
 			break;
-		}
+		default: /* -Wswitch */
 		}
 		UNPROTECT(1);
 		return ans;
 	}
-	case 56: /*         "any" */
-	case 57: /*         "all" */
-	case 58: /*       "anyNA" */
-	case 59: /* "is.unsorted" */
+	case R_FLINT_OPS1_ANY:
+	case R_FLINT_OPS1_ALL:
+	case R_FLINT_OPS1_ANYNA:
+	case R_FLINT_OPS1_ISUNS:
 	{
 		SEXP ans = PROTECT(Rf_allocVector(LGLSXP, 1));
 		int *z = LOGICAL(ans);
 		switch (op) {
-		case 56: /*         "any" */
+		case R_FLINT_OPS1_ANY:
 			for (jx = 0; jx < nx &&  mag_is_zero(x + jx); ++jx) ;
 			z[0] = jx <  nx;
 			break;
-		case 57: /*         "all" */
+		case R_FLINT_OPS1_ALL:
 			for (jx = 0; jx < nx && !mag_is_zero(x + jx); ++jx) ;
 			z[0] = jx >= nx;
 			break;
-		case 58: /*       "anyNA" */
+		case R_FLINT_OPS1_ANYNA:
 			z[0] = 0;
 			break;
-		case 59: /* "is.unsorted" */
+		case R_FLINT_OPS1_ISUNS:
 		{
 			SEXP s_strict = VECTOR_ELT(s_dots, 1);
 			if (XLENGTH(s_strict) == 0)
@@ -1027,48 +1160,50 @@ SEXP R_flint_mag_ops1(SEXP s_op, SEXP s_x, SEXP s_dots)
 			z[0] = jx <  nx;
 			break;
 		}
+		default: /* -Wswitch */
 		}
 		UNPROTECT(1);
 		return ans;
 	}
-	case  3: /*       "is.na" */
-	case  4: /*      "is.nan" */
-	case  5: /* "is.infinite" */
-	case  6: /*   "is.finite" */
-	case  7: /*           "!" */
+	case R_FLINT_OPS1_ISNA:
+	case R_FLINT_OPS1_ISNAN:
+	case R_FLINT_OPS1_ISINF:
+	case R_FLINT_OPS1_ISNUM:
+	case R_FLINT_OPS1_NOT:
 	{
 		ERROR_TOO_LONG(nz, R_XLEN_T_MAX);
 		SEXP ans = PROTECT(Rf_allocVector(LGLSXP, (R_xlen_t) nz));
 		int *z = LOGICAL(ans);
 		switch (op) {
-		case  3: /*       "is.na" */
-		case  4: /*      "is.nan" */
+		case R_FLINT_OPS1_ISNA:
+		case R_FLINT_OPS1_ISNAN:
 			for (jz = 0; jz < nz; ++jz)
 				z[jz] = 0;
 			break;
-		case  5: /* "is.infinite" */
+		case R_FLINT_OPS1_ISINF:
 			for (jz = 0; jz < nz; ++jz)
 				z[jz] = mag_is_inf(x + jz) != 0;
 			break;
-		case  6: /*   "is.finite" */
+		case R_FLINT_OPS1_ISNUM:
 			for (jz = 0; jz < nz; ++jz)
 				z[jz] = mag_is_finite(x + jz) != 0;
 			break;
-		case  7: /*           "!" */
+		case R_FLINT_OPS1_NOT:
 			for (jz = 0; jz < nz; ++jz)
 				z[jz] = mag_is_zero(x + jz) != 0;
 			break;
+		default: /* -Wswitch */
 		}
 		setDDNN1(ans, s_x);
 		UNPROTECT(1);
 		return ans;
 	}
-	case 60: /*     "colSums" */
-	case 61: /*     "rowSums" */
-	case 62: /*    "colMeans" */
-	case 63: /*    "rowMeans" */
+	case R_FLINT_OPS1_COLSUM:
+	case R_FLINT_OPS1_ROWSUM:
+	case R_FLINT_OPS1_COLMEAN:
+	case R_FLINT_OPS1_ROWMEAN:
 	{
-		int byrow = op == 61 || op == 63, domean = op == 62 || op == 63;
+		int byrow = (info & 1) != 0, domean = (info & 2) != 0;
 
 		SEXP dimx = PROTECT(R_do_slot(s_x, R_flint_symbol_dim));
 		if (dimx == R_NilValue || XLENGTH(dimx) < 2)
@@ -1118,26 +1253,25 @@ SEXP R_flint_mag_ops1(SEXP s_op, SEXP s_x, SEXP s_dots)
 		}
 		PROTECT(dimnamesz);
 
-		SEXP ans = PROTECT(newObject("mag"));
-		mag_ptr z = (nz) ? flint_calloc(nz, sizeof(mag_t)) : 0;
-		R_flint_set(ans, z, nz, (R_CFinalizer_t) &R_flint_mag_finalize);
+		SEXP ans = PROTECT(newFlint(R_FLINT_CLASS_MAG, 0, nz));
+		mag_ptr z = R_flint_get_pointer(ans);
 		jx = 0;
 		if (byrow) {
 			for (jz = 0; jz < nz; ++jz)
 				mag_zero(z + jz);
 			for (jt = 0; jt < nt; ++jt)
 				for (jz = 0; jz < nz; ++jz, ++jx)
-					WRAP(mag_add, lower, z + jz, z + jz, x + jx);
+					TERN(mag_add, lower, z + jz, z + jz, x + jx);
 			if (domean)
 			for (jz = 0; jz < nz; ++jz)
-				WRAP(mag_div_ui, lower, z + jz, z + jz, nt);
+				TERN(mag_div_ui, lower, z + jz, z + jz, nt);
 		} else {
 			for (jz = 0; jz < nz; ++jz) {
 				mag_zero(z + jz);
 				for (jt = 0; jt < nt; ++jt, ++jx)
-					WRAP(mag_add, lower, z + jz, z + jz, x + jx);
+					TERN(mag_add, lower, z + jz, z + jz, x + jx);
 				if (domean)
-				WRAP(mag_div_ui, lower, z + jz, z + jz, nt);
+				TERN(mag_div_ui, lower, z + jz, z + jz, nt);
 			}
 		}
 		if (ndz > 1)
