@@ -39,10 +39,10 @@ stopifnot( all.equal(x, x),
                      check.attributes = FALSE),
            all.equal(x, 0,
                      check.class      = FALSE),
-          !all.equal(x, ACF(1L)),
-          !all.equal(x, ACF(1L),
+          !all.equal(x, ACF(0)),
+          !all.equal(x, ACF(0),
                      check.attributes = FALSE),
-           all.equal(x, ACF(1L),
+           all.equal(x, ACF(0),
                      check.class      = FALSE),
           !all.equal(x, y, scale = 1),
            all.equal(x, y, scale = 1, tolerance =   1),
