@@ -17,3 +17,8 @@ for (.cl in c("slong", "fmpz", "fmpq", "mag", "arf", "acf",
 prec <- as.integer(cumprod(rep(c(1, 2), c(1L, 15L))))
 e... <- arb_const_e(prec)
 print(e..., digits = 32L)
+
+
+## Row and column labels were not printed for arrays of length zero
+ulong.array(0L, c(0L, 2L))
+ulong.array(0L, c(2L, 0L))
